<?php

namespace Truevo\PaymentGateway\Controller\Adminhtml\Capture;

use Magento\Sales\Model\Order;
use Truevo\PaymentGateway\Controller\Adminhtml\Base\TruevoAction;

class Index extends TruevoAction
{
    /**
     * Process a pre-authorised order currently on hold.
     */
    public function execute()
    {
        // Setup the class variables
        parent::execute();

        // Check to see if the order can be captured
        if ($this->_transactionDetails['transactionType'] !== "AUTHORIZE" && $this->_order->getState() !== Order::STATE_HOLDED) {
            throw new Exception('Order is not in a valid state to capture the payment.');
        }

        // Call Truevo API
        $response = $this->_truevoApi->capturePayment();

        // Responses
        if ($response) {
            if ($response->outcome->status === "RECEIVED") {
                $transactionUniqueIdentifier = $response->transactionUniqueIdentifier;

                $log = $this->buildLogArray('CAPTURE', $response, ['transaction_unique_identifier' => $transactionUniqueIdentifier]);
                $this->logToOrder($log);

                switch ( $this->pollingTransactionStatus($transactionUniqueIdentifier) ) {
                    case 'COMPLETE':
                        $this->_order->setData('state', Order::STATE_PROCESSING);
                        $this->_order->setStatus(Order::STATE_PROCESSING);

                        $this->_order->save();

                        $this->messageManager->addSuccess(__('Truevo payment successfully captured.'));
                        break;
                    case 'DECLINED':
                        $this->_order->setData('state', Order::STATE_CANCELED);
                        $this->_order->setStatus(Order::STATE_CANCELED);

                        $this->_order->save();

                        $this->messageManager->addError(__('Truevo payment has been declined.'));
                        break;
                    default: 
                        // Payment is still pending
                        $this->messageManager->addNotice(__('Truevo payment still pending.'));
                        break;
                }

                return $this->redirectToOrderView($this->_order->getId());
            }
        }

        // Error capturing payment
        $this->messageManager->addError(__('There was an error capturing the Truevo payment.'));
        return $this->redirectToOrderView($this->_order->getId());
    }
}