<?php

namespace Truevo\PaymentGateway\Controller\Adminhtml\Reversal;

use Magento\Sales\Model\Order;
use Truevo\PaymentGateway\Controller\Adminhtml\Base\TruevoAction;

class Index extends TruevoAction
{
    public function execute()
    {
        // Setup the class variables
        parent::execute();

        // Call Truevo API
        $response = $this->_truevoApi->reversal();

        // Responses
        if ($response) {
            if ($response->outcome->status === "RECEIVED") {
                $transactionUniqueIdentifier = $response->transactionUniqueIdentifier;

                $log = $this->buildLogArray('REVERSAL', $response);
                $this->logToOrder($log);

                switch ( $this->pollingTransactionStatus($transactionUniqueIdentifier) ) {
                    case 'COMPLETE':
                        $this->_order->setData('state', Order::STATE_CANCELED);
                        $this->_order->setStatus(Order::STATE_CANCELED);

                        $this->_order->save();

                        // Successfuly reversed payment
                        $this->messageManager->addSuccess(__('Truevo payment successfully reversed.'));
                        break;
                    case 'DECLINED':
                        $this->messageManager->addError(__('Truevo payment declined the reversal.'));
                        break;
                    default: 
                        // Payment is still pending
                        $this->messageManager->addNotice(__('Reversal is still pending.'));
                        break;
                }

                return $this->redirectToOrderView($this->_order->getId());
            }
        }

        // Error reversing payment
        $this->messageManager->addError(__('There was an error reversing the Truevo payment.'));
        return $this->redirectToOrderView($this->_order->getId());
    }
}