<?php
declare(strict_types = 1);

namespace Truevo\PaymentGateway\Controller\Callback;

use Magento\Framework\App\CsrfAwareActionInterface;
use Magento\Framework\App\Request\InvalidRequestException;
use Magento\Framework\Message\ManagerInterface;
use Magento\Sales\Model\Order\Email\Sender\OrderSender;
use Magento\Framework\App\RequestInterface;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\Action\Action as AppAction;
use Magento\Framework\App\Action\Context;
use Magento\Sales\Model\OrderFactory;
use Truevo\PaymentGateway\Logger\Logger;
use Truevo\PaymentGateway\Model\PaymentMethod;
use Magento\Framework\View\Result\PageFactory;
use Truevo\PaymentGateway\Model\TruevoConfig;
use Magento\Checkout\Model\Session;
use Magento\Customer\Model\Session as CustomerSession;
use Psr\Log\LoggerInterface;
use Magento\Sales\Model\Order;
use Magento\Vault\Api\Data\PaymentTokenFactoryInterface;
use Magento\Vault\Model\PaymentToken;
use Magento\Vault\Model\PaymentTokenRepository;
use Magento\Vault\Api\PaymentTokenManagementInterface;
use Magento\Payment\Helper\Data;
use Exception;

class Index extends AppAction implements CsrfAwareActionInterface
{
    /**
    * @var PaymentMethod
    */
    protected $_paymentMethod;

    /**
     * @var Logger
     */
    protected $_truevologger;

    /**
     * @var Session
     */
    protected $_checkoutSession;

    /**
     * @var PageFactory
     */
    protected $_resultPageFactory;

    /**
    * @var Order
    */
    protected $_order;

    /**
    * @var OrderFactory
    */
    protected $_orderFactory;

    /**
    * @var OrderSender
    */
    protected $_orderSender;

    /**
    * @var LoggerInterface
    */
    protected $_logger;

    /**
     * @var ManagerInterface
     */
	private $_messageManager;

    /**
     * @var ScopeConfigInterface
     */
    protected $_scopeConfig;

    /**
     * @var TruevoConfig
     */
    protected $_truevoConfig;

    /**
     * @var CustomerSession
     */
    protected $_customerSession;

    /**
     * @var use PaymentToken;
     */
    protected $_paymentToken;

    /**
     * @var use PaymentTokenRepository;
     */
    protected $_tokenRepository;

    /**
     * @var use PaymentTokenManagementInterface;
     */
    protected $_tokenManagement;

    /**
     * @var use PaymentTokenManagementInterface;
     */
    protected $_data;

    /**
     * Constructor
     *
     * @param Context $context
     * @param OrderFactory $orderFactory
     * @param PaymentMethod $paymentMethod
     * @param OrderSender $orderSender
     * @param ManagerInterface $messageManager
     * @param LoggerInterface $logger
     * @param Session $checkoutSession
     * @param PageFactory $resultPageFactory
     * @param ScopeConfigInterface $scopeConfig
     * @param Logger $truevologger
     */
    public function __construct(
        Context $context,
        OrderFactory $orderFactory,
        PaymentMethod $paymentMethod,
        OrderSender $orderSender,
        ManagerInterface $messageManager,
        LoggerInterface $logger,
        Logger $truevologger,
        Session $checkoutSession,
        PageFactory $resultPageFactory,
        ScopeConfigInterface $scopeConfig,
        TruevoConfig $truevoConfig,
        PaymentToken $paymentToken,
        PaymentTokenRepository $tokenRepository,
        PaymentTokenManagementInterface $tokenManagement,
        CustomerSession $customerSession,
        Data $data
    ) {
        $this->_paymentMethod = $paymentMethod;
        $this->_orderFactory = $orderFactory;
        $this->_client = $this->_paymentMethod->getClient();
        $this->_orderSender = $orderSender;
		$this->_messageManager = $messageManager;
        $this->_logger = $logger;
        $this->_truevologger = $truevologger;
		$this->_checkoutSession = $checkoutSession;
        $this->_resultPageFactory = $resultPageFactory;
        $this->_scopeConfig = $scopeConfig;
        $this->_truevoConfig = $truevoConfig;
        $this->_paymentToken = $paymentToken;
        $this->_tokenRepository = $tokenRepository;
        $this->_tokenManagement = $tokenManagement;
        $this->_customerSession = $customerSession;
        $this->_data = $data;

        parent::__construct($context);
    }

    public function execute()
    {
        try {
			$this->_success();
			$this->paymentAction();
        } catch (Exception $e) {
            $this->_logger->error($e);
            return $this->_failure();
        }
    }

    public function getOrderId()
    {
        return $this->_objectManager->get('Magento\Checkout\Model\Session')->getLastRealOrderId();
    }

	protected function paymentAction()
	{
        if ($_SERVER['REQUEST_METHOD'] === 'POST') {
            $orderId = $this->getOrderId();
            $this->_loadOrder($orderId);
            $authorized = false;

            $responseId = $_POST['rrn'];
            $responseType = $_POST['transactionType'];
            $responseAmount = $this->_order->getBaseGrandTotal();
            $responseMerchantReference = $_POST['merchantReference'];
            $responseTransactionId = $_POST['transactionId'];
            $responseTransactionStatus = $_POST['transactionStatus'];
            $responseCode = $_POST['responseCode'];

            $message  = 'Transaction Type: ' . strtoupper($responseType) . '<br/>';
            $message .= 'RRN: ' . $responseId . '<br/>';
            $message .= 'Transaction ID: ' . $responseTransactionId . '<br/>';
            $message .= 'Transaction status: ' . $responseTransactionStatus . '<br/>';
            $message .= 'Response Code: ' . $responseCode . '<br/>';
            $message .= 'Merchant Reference: ' .$responseMerchantReference. '<br/>';

            if ( $responseTransactionStatus === 'COMPLETE') {
                if ($_POST['transactionType'] === "AUTHORIZE" || $this->_truevoConfig->isPreauthEnabled()) {
                    $this->_order->setData('state', Order::STATE_HOLDED);
                    $this->_order->setStatus(Order::STATE_HOLDED);
                }

                $this->_order->setBaseTotalPaid($responseAmount);
                $this->_order->setTotalPaid($responseAmount);
                $this->_registerPaymentCapture($responseTransactionId, $responseAmount, $message);

                $redirectUrl = $this->_paymentMethod->getSuccessUrl();
                $this->_redirect($redirectUrl);

            } else if ($_POST['transactionStatus'] === 'PENDING') {
                $this->_registerPaymentCapture($responseTransactionId, $responseAmount, $message);

                $redirectUrl = $this->_paymentMethod->getSuccessUrl();
                $this->_redirect($redirectUrl);
            } else {
                $this->_order->registerCancellation($message)->save();
                
                $redirectUrl = $this->_paymentMethod->getCancelUrl();
                $this->_redirect($redirectUrl);
            }
        } else {
            $this->_messageManager->addErrorMessage(__('Your transaction failed or has been cancelled!'));
            $this->_redirect('checkout/cart');
        }
    }

    protected function _loadOrder($orderId)
    {
        $this->_order = $this->_orderFactory->create()->loadByIncrementId($orderId);

        if (!$this->_order && $this->_order->getId()) {
            throw new Exception('Could not find Magento order with id ' . $orderId);
        }
    }

    protected function _registerPaymentCapture($transactionId, $amount, $message)
    {
        $payment = $this->_order->getPayment();

        $payment->setTransactionId($transactionId)
                ->setPreparedMessage($this->_createTruevoPaymentsComment($message))
                ->setShouldCloseParentTransaction(false)
                ->setIsTransactionClosed(0)
                ->registerCaptureNotification(
                    $amount,
                    true
                );

        $this->_order->save();

        $invoice = $payment->getCreatedInvoice();
        if ($invoice && !$this->_order->getEmailSent()) {
            $this->_orderSender->send($this->_order);
            $this->_order->addStatusHistoryComment(
                __('You notified customer about invoice #%1.', $invoice->getIncrementId())
            )->setIsCustomerNotified(
                true
            )->save();
        }
    }

    protected function _success()
    {
        $this->getResponse()->setStatusHeader(200);
    }

    protected function _failure()
    {
        $this->getResponse()->setStatusHeader(400);
    }

    protected function _createTruevoPaymentsComment($message = '')
    {
        if ($message != '') {
            $message = $this->_order->addStatusHistoryComment($message);
            $message->setIsCustomerNotified(null);
        }

        return $message;
    }

    /**
     * @param RequestInterface $request
     * @return InvalidRequestException|null
     */
    public function createCsrfValidationException(RequestInterface $request): ?InvalidRequestException
    {
        return null;
    }

    /**
     * @param RequestInterface $request
     * @return boolean|null
     */
    public function validateForCsrf(RequestInterface $request): ?bool
    {
        return true;
    }
}
