<?php
declare (strict_types = 1);

namespace Truevo\PaymentGateway\Controller\Cancel;

use Magento\Checkout\Model\Session;
use Magento\Framework\App\Action\Action as AppAction;
use Magento\Framework\App\Action\Context;
use Magento\Framework\App\CsrfAwareActionInterface;
use Magento\Framework\App\RequestInterface;
use Magento\Framework\App\Request\InvalidRequestException;
use Magento\Sales\Model\Order;
use Truevo\PaymentGateway\Model\PaymentMethod;

class Index extends AppAction implements CsrfAwareActionInterface
{
    /**
     * @var PaymentMethod
     */
    protected $_paymentMethod;

    /**
     * @var Session
     */
    protected $_checkoutSession;

    public function __construct(
        Context $context,
        PaymentMethod $paymentMethod,
        Session $checkoutSession
    ) {
        $this->_paymentMethod   = $paymentMethod;
        $this->_checkoutSession = $checkoutSession;

        return parent::__construct($context);
    }

    public function execute()
    {
        $order = $this->_checkoutSession->getLastRealOrder();

        $order->setData('state', Order::STATE_CANCELED);
        $order->setStatus(Order::STATE_CANCELED);
        $order->registerCancellation("Order was cancelled by user")->save();

        $redirectUrl = $this->_paymentMethod->getCancelUrl();
        $this->_redirect($redirectUrl);
    }

    /**
     * @param RequestInterface $request
     * @return InvalidRequestException|null
     */
    public function createCsrfValidationException(RequestInterface $request): ?InvalidRequestException
    {
        return null;
    }

    /**
     * @param RequestInterface $request
     * @return boolean|null
     */
    public function validateForCsrf(RequestInterface $request): ?bool
    {
        return true;
    }
}
