<?php
declare(strict_types = 1);

namespace Truevo\PaymentGateway\Controller\Checkout;

use Magento\Checkout\Model\Session;
use Magento\Sales\Model\Order;
use Magento\Framework\App\Action\Context;
use Magento\Framework\App\Action\Action;
use Magento\Framework\View\Result\PageFactory;
use Truevo\PaymentGateway\Model\PaymentMethod;
use Truevo\PaymentGateway\Logger\Logger;

class Index extends Action
{
    /**
     * @var Session
     */
    protected $_checkoutSession;

    /**
     * @var PageFactory
     */
	protected $_resultPageFactory;

    /**
     * @var Logger
     */
    protected $_logger;

    /**
     * Constructor
     *
     * @param Context $context
     * @param Session $checkoutSession
     * @param PageFactory $resultPageFactory
     * @param PaymentMethod $paymentMethod
     * @param Logger $logger
     */
    public function __construct(
        Context $context,
        Session $checkoutSession,
        PageFactory $resultPageFactory,
        PaymentMethod $paymentMethod,
        \Magento\Framework\Registry $coreRegistry,
        Logger $logger
    ) {
        $this->_coreRegistry = $coreRegistry;
        $this->_paymentMethod = $paymentMethod;
        $this->_checkoutSession = $checkoutSession;
		$this->_resultPageFactory = $resultPageFactory;
        $this->_logger = $logger;
        parent::__construct($context);
    }

    public function execute()
    {
        $html = $this->_paymentMethod->getPostHTML($this->getOrder());

        /** @var \Magento\Framework\View\Result\Page $resultPage */
        $resultPage = $this->_resultPageFactory->create();
        $this->_coreRegistry->register('truevo_widget', $html);
        $this->_logger->info('order ID: ' . $this->getOrder()->getId());

        return $resultPage;
    }

    /**
    * Get order object.
    *
    * @returnOrder
    */
    protected function getOrder()
    {
        return $this->_checkoutSession->getLastRealOrder();
    }
}
