<?php

namespace Truevo\PaymentGateway\Helper;

use Magento\Framework\App\Helper\AbstractHelper;
use Magento\Framework\App\Helper\Context;
use Magento\Framework\Encryption\EncryptorInterface;
use Magento\Store\Model\ScopeInterface;
use Magento\Store\Model\StoreManagerInterface;
use Magento\Framework\Module\ModuleListInterface;
use Magento\Framework\Module\StatusFactory;
use Truevo\PaymentGateway\Model\TruevoConfig;
use Truevo\PaymentGateway\Logger\Logger;

/**
 * @SuppressWarnings(PHPMD.ExcessivePublicCount)
 */
class Data extends AbstractHelper
{

    const TRUEVO_ACTIVE = 'payment/truevo_card_payment/active';

    /**
     * @var EncryptorInterface
     */
    private $encryptor;

    /**
     * @var StoreManagerInterface
     */
    private $storeManager;

    /**
     * @var ModuleListInterface
     */
    private $moduleList;

    /**
     * @var StatusFactory
     */
    private $moduleStatusFactory;

    /**
     * @var Config
     */
    private $config;

    /**
     * @var Logger
     */
    private $truevologger;

    /**
     * Data constructor.
     *
     * @param ModuleListInterface $moduleList
     * @param Context $context
     * @param EncryptorInterface $encryptor
     * @param StoreManagerInterface $storeManager
     * @param StatusFactory $moduleStatusFactory
     * @param AmazonConfig $config
     * @param Logger $truevologger
     */
    public function __construct(
        ModuleListInterface $moduleList,
        Context $context,
        EncryptorInterface $encryptor,
        StoreManagerInterface $storeManager,
        StatusFactory $moduleStatusFactory,
        TruevoConfig $config,
        Logger $truevologger
    ) {
        parent::__construct($context);
        $this->moduleList = $moduleList;
        $this->encryptor = $encryptor;
        $this->storeManager = $storeManager;
        $this->moduleStatusFactory = $moduleStatusFactory;
        $this->config = $config;
        $this->truevologger = $truevologger;
    }

    public function loggingTruevo($data ,$scope = ScopeInterface::SCOPE_STORE, $scopeCode = null) {
        //SET
        $enabled = $this->config->getlogging();

        //PROCESS
        if ($enabled) {
            $this->truevologger->info($data);
        }
    }

    /**
     * @return null
     */
    public function getVersion()
    {
        $version = $this->moduleList->getOne('Truevo_PaymentGateway');
        if ($version && isset($version['setup_version'])) {
            return $version['setup_version'];
        } else {
            return null;
        }
    }

    /**
     * Get the IP address of the client making a request to the server.
     *
     * @return string|null
     */
    public function getClientIpAddress()
    {
        $ipAddress = isset($_SERVER['REMOTE_ADDR']) ? trim($_SERVER['REMOTE_ADDR']) : null;

        // Check IP address is valid and not localhost
        if (filter_var($ipAddress, FILTER_VALIDATE_IP) && (substr($ipAddress, 0, 4) != '127.' && $ipAddress != '::1')) {
            return $ipAddress;
        }

        return null;
    }

    /**
     * Get the trimmed string between two other strings.
     *
     * @param string $string
     * @param string $start
     * @param string $end
     * @return string
     */
    public function getStringBetween($string, $start, $end)
    {
        $string = ' ' . $string;
        $ini    = strpos($string, $start);
        if ($ini == 0) {
            return '';
        }

        $ini += strlen($start);
        $len = strpos($string, $end, $ini) - $ini;
        $substr = substr($string, $ini, $len);

        return trim($substr);
    }

    /**
     * Fix precision error on PHP 7.1 when json encoding
     *
     * @return void
     */
    public function _serializePrecisionFix()
    {
        // json_encode breaks decimals in PHP 7.1 due to the serialize_precision
        if (version_compare(phpversion(), '7.1', '>=')) {
            ini_set( 'serialize_precision', -1 );
        }
    }
}
