<?php

namespace Truevo\PaymentGateway\Model;

use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\Encryption\EncryptorInterface;
use Magento\Framework\View\Asset\Repository;
use Magento\Store\Model\ScopeInterface;
use Magento\Store\Model\StoreManagerInterface;

class TruevoConfig
{
    const SANDBOX_WIDGET_URL    = 'https://sandbox-truevo-drop-in-hosted-payment-widget.s3.eu-west-1.amazonaws.com/merchant.minified.js';
    const PRODUCTION_WIDGET_URL = 'https://pay.truevo.com/widget/merchant.minified.js';

    const SANDBOX_API_URL    = 'https://sandbox.unify.truevo.com';
    const PRODUCTION_API_URL = 'https://unify.truevo.com'; 

    /**
     * @var \Magento\Framework\App\Config\ScopeConfigInterface
     */
    protected $scopeConfig;

    /**
     * @var StoreManagerInterface
     */
    private $storeManager;

    /**
     * @var EncryptorInterface
     */
    protected $_encryptor;

    /**
     * @var Repository
     */
    protected $_assetRepo;

    /**
     * Config constructor.
     *
     * @param StoreManagerInterface $storeManager
     * @param ScopeConfigInterface $scopeConfig
     */
    public function __construct(
        StoreManagerInterface $storeManager,
        ScopeConfigInterface $scopeConfig,
        EncryptorInterface $encryptor,
        Repository $assetRepo
    ) {
        $this->storeManager = $storeManager;
        $this->scopeConfig  = $scopeConfig;
        $this->_encryptor   = $encryptor;
        $this->_assetRepo   = $assetRepo;
    }

    /**
     * @param string $scope
     * @param null $scopeCode
     *
     * @return mixed
     */
    public function getDescription($scope = ScopeInterface::SCOPE_STORE, $scopeCode = null)
    {
        //SET
        $result = $this->scopeConfig->getValue(
            'payment/truevo_card_payment/description',
            $scope,
            $scopeCode
        );

        //RETURN
        return $result;
    }

    /**
     * @return string
     */
    public function getWidgetUrl()
    {
        return (!$this->isSandboxMode()) ? $this->getLocalMerchantScriptUrl() : self::SANDBOX_WIDGET_URL;
    }

    /**
     * @param string $scope
     * @param null $scopeCode
     *
     * @return mixed
     */
    public function getMid($scope = ScopeInterface::SCOPE_STORE, $scopeCode = null)
    {
        //set
        $output = '';

        //processing
        if ($this->isSandboxMode()) {
            $output = $this->scopeConfig->getValue(
                'payment/truevo_payment/test_mid',
                $scope,
                $scopeCode
            );
        } else {
            $output = $this->scopeConfig->getValue(
                'payment/truevo_payment/live_mid',
                $scope,
                $scopeCode
            );
        }

        //return
        return $output;
    }

    /**
     * @param string $scope
     * @param null $scopeCode
     *
     * @return mixed
     */
    public function getTid($scope = ScopeInterface::SCOPE_STORE, $scopeCode = null)
    {
        //set
        $output = '';

        //processing
        if ($this->isSandboxMode()) {
            $output = $this->scopeConfig->getValue(
                'payment/truevo_payment/test_tid',
                $scope,
                $scopeCode
            );
        } else {
            $output = $this->scopeConfig->getValue(
                'payment/truevo_payment/live_tid',
                $scope,
                $scopeCode
            );
        }

        //return
        return $output;
    }

    /**
     * @param string $scope
     * @param null $scopeCode
     *
     * @return mixed
     */
    public function isSandboxMode($scope = ScopeInterface::SCOPE_STORE, $scopeCode = null)
    {
        //set
        $isSandboxMode = $this->scopeConfig->getValue(
            'payment/truevo_payment/sandbox',
            $scope,
            $scopeCode
        );

        //return
        return $isSandboxMode;
    }

    /**
     * @param string $scope
     * @param null $scopeCode
     *
     * @return mixed
     */
    public function showLogo($scope = ScopeInterface::SCOPE_STORE, $scopeCode = null)
    {
        //SET
        $result = $this->scopeConfig->getValue(
            'payment/truevo_payment/showlogo',
            $scope,
            $scopeCode
        );

        //RETURN
        return $result;
    }

    /**
     * @param string $scope
     * @param null $scopeCode
     *
     * @return mixed
     */
    public function getlogging($scope = ScopeInterface::SCOPE_STORE, $scopeCode = null)
    {
        //SET
        $result = $this->scopeConfig->getValue(
            'payment/truevo_payment/logging',
            $scope,
            $scopeCode
        );

        //RETURN
        return $result;
    }

    /**
     * @param string $scope
     * @param null $scopeCode
     *
     * @return mixed
     */
    public function getcss($scope = ScopeInterface::SCOPE_STORE, $scopeCode = null)
    {
        //SET
        $result = $this->scopeConfig->getValue(
            'payment/truevo_card_payment/css',
            $scope,
            $scopeCode
        );

        //RETURN
        return $result;
    }

    /**
     * @param string $scope
     * @param null $scopeCode
     *
     * @return mixed
     */
    public function getPaymentStyle($scope = ScopeInterface::SCOPE_STORE, $scopeCode = null)
    {
        return $this->scopeConfig->getValue(
            'payment/truevo_card_payment/widget_style',
            $scope,
            $scopeCode
        );
    }

    /**
     * @param string $scope
     * @param null $scopeCode
     *
     * @return mixed
     */
    public function getLocale($scope = ScopeInterface::SCOPE_STORE, $scopeCode = null)
    {
        $output = $this->scopeConfig->getValue(
            'payment/truevo_card_payment/locale',
            $scope,
            $scopeCode
        );

        return 0;
    }

    /**
     * Gets customer's current currency
     *
     * @param null $store
     * @return mixed
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    protected function getCurrentCurrencyCode($store = null)
    {
        return $this->storeManager->getStore()->getCurrentCurrency()->getCode();
    }

    /**
     * Retrieves the base currency of the store.
     *
     * @param null $store
     * @return mixed
     */
    public function getBaseCurrencyCode($store = null)
    {
        return $this->scopeConfig->getValue(
            'currency/options/base',
            ScopeInterface::SCOPE_STORE,
            $store
        );
    }

    /**
     * @param string $scope
     * @param null $scopeCode
     *
     * @return mixed
     */
    public function getAuthToken($scope = ScopeInterface::SCOPE_STORE, $scopeCode = null)
    {
        //set
        $output = '';

        //processing
        if ($this->isSandboxMode()) {
            $output = $this->scopeConfig->getValue(
                'payment/truevo_payment/test_api_token',
                $scope,
                $scopeCode
            );
        } else {
            $output = $this->scopeConfig->getValue(
                'payment/truevo_payment/live_api_token',
                $scope,
                $scopeCode
            );
        }

        //return
        return $output;
    }

    public function getTruevoApiUrl()
    {
        return ($this->isSandboxMode()) ? self::SANDBOX_API_URL : self::PRODUCTION_API_URL;
    }

    public function getTransactionType()
    {
        return ($this->isPreauthEnabled()) ? 'authorize' : 'sale';
    }

    public function isPreauthEnabled($scope = ScopeInterface::SCOPE_STORE, $scopeCode = null)
    {
        //SET
        $result = $this->scopeConfig->getValue(
            'payment/truevo_payment/preauthorization',
            $scope,
            $scopeCode
        );

        //RETURN
        return $result;
    }

    public function isTokenizationEnabled($scope = ScopeInterface::SCOPE_STORE, $scopeCode = null)
    {
        //SET
        $result = $this->scopeConfig->getValue(
            'payment/truevo_payment/tokenization',
            $scope,
            $scopeCode
        );

        //RETURN
        return $result;
    }

    public function getFullTruevoEndpoint($endpoint, $queryParams = null)
    {
        //SET
        $url = $this->getTruevoApiUrl();

        //PROCESSING
        if (strpos($endpoint, '/') != 0) {
            $url .= '/';
        }

        $url .= $endpoint;

        if (!is_null($queryParams)) {
            $url .= '?' . $queryParams;
        }

        //RETURN
        return $url;
    }

    public function getLocalMerchantScriptUrl()
    {
        return $this->_assetRepo->getUrl("Truevo_PaymentGateway::js/merchant.minified.js");
    }
}
