<?php
namespace Truevo\PaymentGateway\Plugin\Sales\Block\Adminhtml\Order;

use Magento\Framework\UrlInterface;
use Magento\Framework\AuthorizationInterface;
use Magento\Sales\Block\Adminhtml\Order\View as OrderView;
use Magento\Sales\Model\Order;
use Truevo\PaymentGateway\Model\TruevoAPI;

class View
{
    /** @var \Magento\Framework\UrlInterface */
    protected $_urlBuilder;

    /** @var \Magento\Framework\AuthorizationInterface */
    protected $_authorization;

    public function __construct(
        UrlInterface $url,
        AuthorizationInterface $authorization
    ) {
        $this->_urlBuilder = $url;
        $this->_authorization = $authorization;
    }

    public function beforeSetLayout(OrderView $orderView) {
        $order = $orderView->getOrder();
        
        if ($this->isTruevoPayment($order)) {
            // Truevo API
            $truevoApi = new TruevoAPI($order);
            $response = $truevoApi->getDetails();

            if ($response) {
                // Only show capture button if it is allowed
                if ($response->paymentDetails->isCaptureAllowed && $order->getState() === Order::STATE_HOLDED) {
                    $captureUrl = $this->_urlBuilder->getUrl('truevo/capture/index', ['order_id' => $orderView->getOrderId()]);

                    $orderView->addButton(
                        'order_capture_button',
                        [
                            'label'   => __('Capture Pre-authorisation'),
                            'class'   => __('truevo-capture-button'),
                            'id'      => 'truevo-capture-button',
                            'onclick' => 'setLocation(\'' . $captureUrl . '\')',
                        ]
                    );
                }

                // Only show refund button if it is allowed
                if ($response->paymentDetails->isRefundAllowed && $response->paymentDetails->availableForRefund > 0) {
                    $refundUrl = $this->_urlBuilder->getUrl('truevo/refund/index', ['order_id' => $orderView->getOrderId(), '_query' => ['available_for_refund' => $response->paymentDetails->availableForRefund]]);

                    $orderView->addButton(
                        'order_refund_button',
                        [
                            'label'   => __('Refund Order'),
                            'class'   => __('btn-secondary'),
                            'id'      => 'truevo-capture-button',
                            'onclick' => 'setLocation(\'' . $refundUrl . '\')',
                        ]
                    );
                }

                // Only show reversal button if it is allowed
                if ($response->paymentDetails->isReverseAllowed) {
                    $reversalUrl = $this->_urlBuilder->getUrl('truevo/reversal/index', ['order_id' => $orderView->getOrderId()]);

                    $orderView->addButton(
                        'order_reversal_button',
                        [
                            'label'   => __('Reverse Order'),
                            'class'   => __('btn-secondary'),
                            'id'      => 'truevo-capture-button',
                            'onclick' => 'setLocation(\'' . $reversalUrl . '\')',
                        ]
                    );
                }
            }
        }
    }

    /**
     * Check if the order was done via the Truevo payment method.
     * 
     * @param Order $order
     * @return boolean Was the order paid for via Truevo?
     */
    private function isTruevoPayment(Order $order)
    {
        // Check if this payment method was used
        if ($payment = $order->getPayment()) {
            return $payment->getMethod() === "truevo_card_payment";
        }

        return false;
    }
}
